/****************************************************************************
 *
 * Copyright (c) 2011-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.commons.log;
import javax.xml.namespace.QName;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;
import com.ebmwebsourcing.easycommons.uuid.SimpleUUIDGenerator;

/**
 * Helper for {@link ExecutionContext}.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public final class PetalsExecutionContext {

    /**
     * Name of the contextual property storing the container name
     */
    public static final String CONTAINER_NAME_PROPERTY_NAME = "containerName";

    /**
     * Name of the contextual property storing the flow instance identifier.
     */
    public static final String FLOW_INSTANCE_ID_PROPERTY_NAME = "flowInstanceId";

    /**
     * Name of the contextual property storing the flow step identifier.
     */
    public static final String FLOW_STEP_ID_PROPERTY_NAME = "flowStepId";

    /**
     * Name of the contextual property storing the previous step identifier.
     */
    public static final String FLOW_PREVIOUS_STEP_ID_PROPERTY_NAME = "flowPreviousStepId";

    /**
     * Name of the contextual property storing the interface name of the invoked
     * provider step
     */
    public static final String FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME = "flowStepInterfaceName";

    /**
     * Name of the contextual property storing the operation name of the invoked
     * provider step
     */
    public static final String FLOW_STEP_OPERATION_NAME_PROPERTY_NAME = "flowStepOperationName";

    /**
     * Name of the contextual property storing the service name of the invoked
     * provider step.
     */
    public static final String FLOW_STEP_SERVICE_NAME_PROPERTY_NAME = "flowStepServiceName";

    /**
     * Name of the contextual property storing the endpoint name of the invoked
     * provider step
     */
    public static final String FLOW_STEP_ENDPOINT_NAME_PROPERTY_NAME = "flowStepEndpointName";

    /**
     * Name of the contextual property storing the message exchange
     */
    public static final String FLOW_EXCHANGE_PROPERTY_NAME = "flowExchange";

    /**
     * An UUID generator.
     */
    private static final SimpleUUIDGenerator simpleUUIDGenerator = new SimpleUUIDGenerator();

    /**
     * Set in the {@link ExecutionContext}, the value of
     * {@link PetalsExecutionContext#CONTAINER_NAME_PROPERTY_NAME}.
     * 
     * @param containerName
     *            the container name to set as value of the contextual property
     *            {@link PetalsExecutionContext#CONTAINER_NAME_PROPERTY_NAME}.
     */
    public static final void putContainerName(final String containerName) {
        assert containerName != null;
        ExecutionContext.getProperties().setProperty(CONTAINER_NAME_PROPERTY_NAME, containerName);
    }

    /**
     * Set in the {@link ExecutionContext}, the value of
     * {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}.
     * 
     * @param flowInstanceId
     *            the flow instance identifier to set as value of the contextual
     *            property
     *            {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}.
     */
    public static final void putFlowInstanceId(String flowInstanceId) {
        assert flowInstanceId != null;
        ExecutionContext.getProperties()
                .setProperty(FLOW_INSTANCE_ID_PROPERTY_NAME, flowInstanceId);
    }

    /**
     * Initialize the the value of
     * {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}
     */
    public static final void initFlowInstanceId() {
        putFlowInstanceId(simpleUUIDGenerator.getNewID());
    }

    public static final void putFlowStepId(String flowStepId) {
        assert flowStepId != null;
        ExecutionContext.getProperties().setProperty(FLOW_STEP_ID_PROPERTY_NAME, flowStepId);
    }

    /**
     * Set previous and current stepId in the {@link ExecutionContext} Only
     * provider mode.
     */
    public static void nextFlowStepId() {
        if (ExecutionContext.getProperties().getProperty(FLOW_STEP_ID_PROPERTY_NAME) != null) {
            ExecutionContext.getProperties().setProperty(FLOW_PREVIOUS_STEP_ID_PROPERTY_NAME,
                    ExecutionContext.getProperties().getProperty(FLOW_STEP_ID_PROPERTY_NAME));
        }
        ExecutionContext.getProperties().setProperty(FLOW_STEP_ID_PROPERTY_NAME,
                simpleUUIDGenerator.getNewID());
    }

    public static final void putFlowStepInterfaceName(QName interfaceName) {
        assert interfaceName != null;
        ExecutionContext.getProperties().setProperty(FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME,
                interfaceName.toString());
    }

    public static final void putFlowStepOperationName(QName operation) {
        assert operation != null;
        ExecutionContext.getProperties().setProperty(FLOW_STEP_OPERATION_NAME_PROPERTY_NAME,
                operation.toString());
    }

    public static final void putFlowStepServiceName(QName serviceName) {
        assert serviceName != null;
        ExecutionContext.getProperties().setProperty(FLOW_STEP_SERVICE_NAME_PROPERTY_NAME,
                serviceName.toString());
    }

    public static final void putFlowStepEndpointName(String flowStepEndpointName) {
        assert flowStepEndpointName != null;
        ExecutionContext.getProperties().setProperty(FLOW_STEP_ENDPOINT_NAME_PROPERTY_NAME,
                flowStepEndpointName);
    }

    /**
     * Remove all properties except the container name
     */
    public static final void clear() {
        final String containerName = ExecutionContext.getProperties().getProperty(
                PetalsExecutionContext.CONTAINER_NAME_PROPERTY_NAME);
        ExecutionContext.getProperties().clear();
        if (containerName != null) {
            PetalsExecutionContext.putContainerName(containerName);
        }

    }

    public final static FlowAttributes getFlowAttributes() {
        String flowInstanceId = ExecutionContext.getProperties().getProperty(
                FLOW_INSTANCE_ID_PROPERTY_NAME);
        if (flowInstanceId == null) {
            return null;
        }
        String flowStepId = ExecutionContext.getProperties()
                .getProperty(FLOW_STEP_ID_PROPERTY_NAME);
        if (flowStepId == null) {
            return null;
        }
        return new FlowAttributes(flowInstanceId, flowStepId);
    }

    public static final void putFlowAttributes(FlowAttributes flowAttributes) {
        assert flowAttributes != null;
        PetalsExecutionContext.putFlowInstanceId(flowAttributes.getFlowInstanceId());
        PetalsExecutionContext.putFlowStepId(flowAttributes.getFlowStepId());
    }
}
